'use strict';

const _ = require('lodash');
const path = require('path');
const fs = require('fs');
const debug = require('debug')('adonis:framework');
const yaml = require('js-yaml');

const AdonisEnv = require('@adonisjs/framework/src/Env');

class Env extends AdonisEnv {
    _interpolate (env, envConfig) {
        if (typeof env === 'string') {
            const matches = env.match(/\$([a-zA-Z0-9_]+)|\${([a-zA-Z0-9_]+)}/g) || [];

            _.each(matches, (match) => {
                const key = match.replace(/\$|{|}/g, '');
                const variable = envConfig[key] || process.env[key] || '';
                env = env.replace(match, this._interpolate(variable));
            });
        }

        return env;
    }

    getEnvPath () {
        if (!process.env.ENV_PATH || process.env.ENV_PATH.length === 0) {
            return 'environment.yaml';
        }
        return process.env.ENV_PATH;
    }

    load (filePath, overwrite = true, encoding = 'utf8') {
        const options = {
            path: path.isAbsolute(filePath) ? filePath : path.join(this.appRoot, filePath),
            encoding,
        };
        try {
            const envConfig = yaml.safeLoad(fs.readFileSync(options.path, options.encoding));

            debug('%s environment file from %s', overwrite ? 'merging' : 'loading', options.path);

            _.each(envConfig, (value, key) => {
                if (process.env[key] === undefined || overwrite) {
                    process.env[key] = this._interpolate(value, envConfig);
                }
            });

            return { parsed: envConfig }
        } catch (error) {
            return { error }
        }
    }
}

module.exports = Env;
